(*----------------------------------------------------------------------------*)
(* Simulation der Dynamik eines Assoziativspeichers zur Herleitung der        *)
(* Kapazitaet des Arbeitsgedaechtnisses (7+-2-Regel).                         *)
(* Joachim Pimiskern, 30.6.2001                                               *)
(*----------------------------------------------------------------------------*)
unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;


const
  cXFakt = 5;
  cYFakt = 4;

type
  TFormWorkmem = class(TForm)
    ButtonStart: TButton;
    Memo: TMemo;
    procedure ButtonStartClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    simTime     : integer;
    countOfUnits: integer;
    patternUnits: integer;
    Ende        : boolean;

    old1x,
    old1y,
    old2x,
    old2y,
    old3x,
    old3y: integer;
    procedure Simulate(var Richtig,Falsch,Unentschieden: integer);
    procedure writeData(Bruch: double; Richtig,Falsch,Unentschieden: integer);
  public
    procedure main;
    procedure PaintGrid;
  end;

var
  FormWorkmem: TFormWorkmem;

implementation

{$R *.DFM}







(*----------------------------------------------------------------------------*)
(* Ein Erinnerungsschritt. Diese Prozedur beschreibt die Zustandsuebergaenge  *)
(* des Assoziativspeichers, nachdem ein Teil der "Flaeche" mit einem Teil des *)
(* Zielmusters initialisiert wurde. Die Anzahl der Simulationsschritte ist    *)
(* willkuerlich festgesetzt simTime * countOfUnits, aber groessere Zeiten     *)
(* bringen i.a. keine wesentliche Aenderung des Ergebnisses. Die Simulation   *)
(* wird vorzeitig beendet, wenn patternUnits auf 0 oder auf countOfUnits      *)
(* gesetzt wird, was einerseits bedeutet, dass das urspruengliche Muster      *)
(* komplett verdraengt wurde oder dass es sich durchgesetzt hat.              *)
(*----------------------------------------------------------------------------*)
procedure TFormWorkmem.Simulate(var Richtig,Falsch,Unentschieden: integer);
var i: integer;
begin
  i := 0;
  while ((i < simTime * countOfUnits) and
         (patternUnits > 0) and
         (patternUnits < countOfUnits)) do
  begin
    (*--- Musterneuron gewaehlt ---*)
    if (random <= patternUnits / countOfUnits) then
       begin
         if (random > patternUnits / countOfUnits) then
            if (random > 0.5) then
               patternUnits := patternUnits - 1;
       end
    else
    (*--- Zufallsneuron gewaehlt ---*)
       begin
         if (random <= patternUnits / countOfUnits) then
            patternUnits := patternUnits + 1
         else
         if (random > 0.5) then
            patternUnits := patternUnits + 1
         else
            patternUnits := patternUnits - 1;
       end;

    i := i + 1;
  end;

  if (patternUnits <= 0) then
     Falsch := Falsch + 1
  else
  if (patternUnits >= countOfUnits) then
     Richtig := Richtig + 1
  else
     Unentschieden := Unentschieden + 1;
end;

procedure TFormWorkmem.ButtonStartClick(Sender: TObject);
begin
  PaintGrid;
  main;
end;


(*----------------------------------------------------------------------------*)
(* Fuer verschiedene Brueche fraction von 0.0 bis 1.0 werden 100 Simulationen *)
(* durchgefuehrt. Bei jedem der Brueche wird ein derart grosser Anteil der    *)
(* Gesamtneuronenzahl mit einem Teil des Zielmusters initialisiert. Der Rest  *)
(* der Neuronen wird auf einen Zufallswert gesetzt. Diese Initialisierungen   *)
(* werden nur fiktiv vorgenommen. Nur die Anzahlen der gesetzten Neuronen     *)
(* sind hier interessant. Nach den Simulationen wird ausgegeben, in wie       *)
(* vielen Faellen alle Neuronen (countOfUnits) durch das urspruengliche       *)
(* Muster (patternUnits) erobert werden konnten.                              *)
(*----------------------------------------------------------------------------*)
procedure TFormWorkmem.main;
var Richtig,
    Falsch,
    Unentschieden: integer;
    j            : integer;
    fraction     : double;
begin
  countOfUnits  := 1000;
  simTime       := 50 * countOfUnits;

  Ende := false;
  fraction := 0.0;
  while ((fraction <= 1.0) and not Ende) do
  begin
    Richtig       := 0;
    Falsch        := 0;
    Unentschieden := 0;
    for j := 0 to 99 do
        begin
          patternUnits  := trunc(countOfUnits * fraction);
          Simulate(Richtig,Falsch,Unentschieden);
        end;
    writeData(fraction,Richtig,Falsch,Unentschieden);
    Memo.Update;
    fraction := fraction + 0.005;
  end;
end;

procedure TFormWorkmem.writeData(Bruch: double; Richtig,Falsch,Unentschieden: integer);
begin
  Memo.Lines.Add(Format('%10.2f %3d %3d %3d',[Bruch,Richtig,Falsch,Unentschieden]));


  Canvas.Pen.Color := clGreen;
  Canvas.Pen.Width := 2;
  Canvas.MoveTo(old3x,old3y);
  old3x := trunc(cXFakt * 100*Bruch);
  old3y := cYFakt * (100-Unentschieden);
  Canvas.LineTo(old3x,old3y);

  Canvas.Pen.Color := clBlue;
  Canvas.Pen.Width := 2;
  Canvas.MoveTo(old1x,old1y);
  old1x := trunc(cXFakt * 100*Bruch);
  old1y := cYFakt * (100-Richtig);
  Canvas.LineTo(old1x,old1y);

  Canvas.Pen.Color := clRed;
  Canvas.Pen.Width := 2;
  Canvas.MoveTo(old2x,old2y);
  old2x := trunc(cXFakt * 100*Bruch);
  old2y := cYFakt * (100-Falsch);
  Canvas.LineTo(old2x,old2y);
end;

procedure TFormWorkmem.FormCreate(Sender: TObject);
begin
  Old1x := -1000;
  Old1y := 0;
  Old2x := -1000;
  Old2y := 0;
  Old3x := -1000;
  Old3y := 0;
end;


procedure TFormWorkmem.PaintGrid;
var i: integer;
begin
  Canvas.Pen.Color := clBlack;

  i := 0;
  while (i <= 100) do
  begin
    Canvas.MoveTo(cXFakt * i,0);
    Canvas.LineTo(cXFakt * i,cYFakt * 100);
    i := i + 10;
  end;

  i := 0;
  while (i <= 100) do
  begin
    Canvas.MoveTo(0           ,cYFakt * i);
    Canvas.LineTo(cXFakt * 100,cYFakt * i);
    i := i + 10;
  end;
end;

end.
